@@ -27,12 +27,9 @@ pub(crate) mod sort;
 pub(crate) mod spark_plan;
 pub(crate) mod util;
 pub use datafusion_comet_spark_expr::timezone;
+mod memory_pools;
 pub(crate) mod utils;
 
-mod fair_memory_pool;
-mod memory_pool;
-pub use memory_pool::*;
-
 #[cfg(test)]
 mod tests {
     #[test]
