@@ -15,10 +15,12 @@
 // specific language governing permissions and limitations
 // under the License.
 
+mod chr;
 mod prediction;
 mod string_space;
 mod substring;
 
+pub use chr::SparkChrFunc;
 pub use prediction::*;
 pub use string_space::StringSpaceExpr;
 pub use substring::SubstringExpr;
